# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Jaepil Jeong

# Based on part of the tornado framework

# Fake unicode literal support:  Python 3.2 doesn't have the u'' marker for
# literal strings, and alternative solutions like "from __future__ import
# unicode_literals" have other problems (see PEP 414).  u() can be applied
# to ascii strings that include \u escapes (but they must not contain
# literal non-ascii characters).
if type('') is not type(b''):
    def u(s):
        return s
    bytes_type = bytes
    unicode_type = str
    basestring_type = str
else:
    def u(s):
        return s.decode('unicode_escape')
    bytes_type = str
    unicode_type = unicode
    basestring_type = basestring


_UTF8_TYPES = (bytes_type, type(None))


def _utf8(value):
    """Converts a string argument to a byte string.

    If the argument is already a byte string or None, it is returned unchanged.
    Otherwise it must be a unicode string and is encoded as utf8.
    """
    if isinstance(value, _UTF8_TYPES):
        return value
    if not isinstance(value, unicode_type):
        raise TypeError(
            "Expected bytes, unicode, or None; got %r" % type(value)
        )
    return value.encode("utf-8")


_TO_UNICODE_TYPES = (unicode_type, type(None))


def _unicode(value):
    """Converts a string argument to a unicode string.

    If the argument is already a unicode string or None, it is returned
    unchanged.  Otherwise it must be a byte string and is decoded as utf8.
    """
    if isinstance(value, _TO_UNICODE_TYPES):
        return value
    if not isinstance(value, bytes_type):
        raise TypeError(
            "Expected bytes, unicode, or None; got %r" % type(value)
        )
    return value.decode("utf-8")


# When dealing with the standard library across python 2 and 3 it is
# sometimes useful to have a direct conversion to the native string type
if str is unicode_type:
    native_str = _unicode
else:
    native_str = _utf8

_BASESTRING_TYPES = (basestring_type, type(None))


def to_utf8(obj):
    """Walks a simple data structure, converting unicode to byte string.

    Supports lists, tuples, and dictionaries.
    """
    if isinstance(obj, unicode_type):
        return _utf8(obj)
    elif isinstance(obj, dict):
        return dict((to_utf8(k), to_utf8(v)) for (k, v) in obj.items())
    elif isinstance(obj, list):
        return list(to_utf8(i) for i in obj)
    elif isinstance(obj, tuple):
        return tuple(to_utf8(i) for i in obj)

    return obj


def to_unicode(obj):
    """Walks a simple data structure, converting byte strings to unicode.

    Supports lists, tuples, and dictionaries.
    """
    if isinstance(obj, bytes_type):
        return _unicode(obj)
    elif isinstance(obj, dict):
        return dict((to_unicode(k), to_unicode(v)) for (k, v) in obj.items())
    elif isinstance(obj, list):
        return list(to_unicode(i) for i in obj)
    elif isinstance(obj, tuple):
        return tuple(to_unicode(i) for i in obj)

    return obj



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Jaepil Jeong

import os
import imp

from collections import namedtuple

import jpype

from escape import to_unicode, to_utf8


def _init_jvm():
    if not jpype.isJVMStarted():
        jars = []
        for top, dirs, files in os.walk(imp.find_module("twkorean")[1] + "/data/lib"):
            for nm in files:
                jars.append(os.path.join(top, nm))
        jpype.startJVM(jpype.getDefaultJVMPath(),
                       "-Djava.class.path=%s" % os.pathsep.join(jars))

_init_jvm()


_TwitterKoreanProcessorBuilder = jpype.JClass(
    "com.twitter.penguin.korean.TwitterKoreanProcessorJava$Builder")

KoreanToken = namedtuple("KoreanToken", ["text", "pos", "unknown"])
KoreanSegment = namedtuple("KoreanSegment", ["start", "length", "token"])
KoreanSegmentWithText = namedtuple("KoreanSegmentWithText", ["text", "segments"])
StemmedTextWithTokens = namedtuple("StemmedTextWithTokens", ["text", "tokens"])


class TwitterKoreanProcessor(object):
    def __init__(self, normalization=True, stemming=True):
        super(TwitterKoreanProcessor, self).__init__()

        builder = _TwitterKoreanProcessorBuilder()
        if not normalization:
            builder.disableNormalizer()
        if not stemming:
            builder.disableStemmer()

        self._processor = builder.build()

    def normalize(self, text):
        encode = lambda t: jpype.java.lang.String(t) if isinstance(text, unicode)\
            else jpype.java.lang.String(to_unicode(t))
        decode = lambda t: t if isinstance(text, unicode) else to_utf8(t)

        return decode(self._processor.normalize(encode(text)))

    # def stem(self, text):
    #     pass

    def tokenize(self, text):
        encode = lambda t: jpype.java.lang.String(t) if isinstance(text, unicode)\
            else jpype.java.lang.String(to_unicode(t))
        decode = lambda t: t if isinstance(text, unicode) else to_utf8(t)

        tokens = self._processor.tokenize(encode(text))
        return [
            KoreanToken(
                text=decode(t.text()), pos=decode(t.pos().toString()), unknown=t.unknown()
            ) for t in tokens
        ]

    def tokenize_to_strings(self, text):
        encode = lambda t: jpype.java.lang.String(t) if isinstance(text, unicode)\
            else jpype.java.lang.String(to_unicode(t))
        decode = lambda t: t if isinstance(text, unicode) else to_utf8(t)

        tokens = self._processor.tokenizeToStrings(encode(text))
        return [decode(t) for t in tokens]

    def tokenize_with_index(self, text):
        encode = lambda t: jpype.java.lang.String(t) if isinstance(text, unicode)\
            else jpype.java.lang.String(to_unicode(t))
        decode = lambda t: t if isinstance(text, unicode) else to_utf8(t)

        result = []
        tokens = self._processor.tokenizeWithIndex(encode(text))
        for t in tokens:
            token = KoreanToken(text=decode(t.token().text()),
                                pos=decode(t.token().pos().toString()),
                                unknown=t.token().unknown())
            segment = KoreanSegment(start=t.start(), length=t.length(), token=token)
            result.append(segment)

        return result

    # def tokenize_with_index_with_stemmer(self, text):
    #     encode = lambda t: jpype.java.lang.String(t) if isinstance(text, unicode)\
    #         else jpype.java.lang.String(to_unicode(t))
    #     decode = lambda t: t if isinstance(text, unicode) else to_utf8(t)

    #     token = self._processor.tokenizeWithIndexWithStemmer(encode(text))
    #     segments = []
    #     for segment in token.segments():
    #         token = KoreanToken(text=decode(segment.token().text()),
    #                             pos=decode(segment.token().pos().toString()),
    #                             unknown=segment.token().unknown())
    #         segment = KoreanSegment(start=segment.start(), length=segment.length(), token=token)
    #         segments.append(segment)

    #     return KoreanSegmentWithText(text=decode(token.text()), segments=segments)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Jaepil Jeong

from __future__ import print_function

from twkorean import TwitterKoreanProcessor


def print_tokens(tokens, end="\n"):
    if isinstance(tokens, list):
        print("[", end="")
    elif isinstance(tokens, tuple):
        print("(", end="")

    for t in tokens:
        if t != tokens[-1]:
            elem_end = ", "
        else:
            elem_end = ""

        if isinstance(t, (list, tuple)):
            print_tokens(t, end=elem_end)
        else:
            print(t, end=elem_end)

    if isinstance(tokens, list):
        print("]", end=end)
    elif isinstance(tokens, tuple):
        print(")", end=end)


text = u"한국어를 처리하는 예시입니닼ㅋㅋㅋㅋㅋ"

# Tokenize with normalization + stemmer
processor = TwitterKoreanProcessor()
# output: [한국어, 를, 처리, 하다, 예시, 이다, ㅋㅋ]
tokens = processor.tokenize_to_strings(text)
print_tokens(tokens)

# output: [
#     (한국어, Noun, 0), (를, Josa, 0), (처리, Noun, 0), (하다, Verb, 0),
#     (예시, Noun, 0), (이다, Adjective, 0), (ㅋㅋ, KoreanParticle, 0)
# ]
tokens = processor.tokenize(text)
print_tokens(tokens)


# Tokenize without stemmer
processor = TwitterKoreanProcessor(stemming=False)
# output: [한국어, 를, 처리, 하는, 예시, 입니, 다, ㅋㅋ]
tokens = processor.tokenize_to_strings(text)
print_tokens(tokens)

# output: [
#     (한국어, Noun, 0), (를, Josa, 0), (처리, Noun, 0), (하는, Verb, 0),
#     (예시, Noun, 0), (입니, Adjective, 0), (다, Eomi, 0), (ㅋㅋ, KoreanParticle, 0)
# ]
tokens = processor.tokenize(text)
print_tokens(tokens)


# Tokenize with neither normalization nor stemmer
processor = TwitterKoreanProcessor(normalization=False, stemming=False)
# output: [한국어, 를, 처리, 하는, 예시, 입니, 닼, ㅋㅋㅋㅋㅋ]
tokens = processor.tokenize_to_strings(text)
print_tokens(tokens)

# output: [
#     (한국어, Noun, 0), (를, Josa, 0), (처리, Noun, 0), (하는, Verb, 0),
#     (예시, Noun, 0), (입니, Adjective, 0), (닼, Noun, 1), (ㅋㅋㅋㅋㅋ, KoreanParticle, 0)
# ]
tokens = processor.tokenize(text)
print_tokens(tokens)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014 Jaepil Jeong

from __future__ import print_function

import os

from setuptools import setup
from setuptools.command.install import install

try:
    from urllib import urlretrieve
except:
    from urllib.request import urlretrieve


__VERSION__ = "0.1.2"

_JAVA_LIB_URLS = [
    "https://repo1.maven.org/maven2/org/scala-lang/scala-library/2.11.4/scala-library-2.11.4.jar",
    "https://repo1.maven.org/maven2/org/apache/thrift/libthrift/0.9.2/libthrift-0.9.2.jar",
    "https://repo1.maven.org/maven2/org/yaml/snakeyaml/1.14/snakeyaml-1.14.jar",
    "https://repo1.maven.org/maven2/com/twitter/twitter-text/1.10.2/twitter-text-1.10.2.jar",
    "https://repo1.maven.org/maven2/com/twitter/penguin/korean-text/2.3.3/korean-text-2.3.3.jar",
]
_PATH_TO_LIB = os.path.join(os.path.abspath(os.path.dirname((__file__))), "twkorean/data/lib")


class InstallCommand(install):
    @staticmethod
    def download_jars(target_path):
        for url in _JAVA_LIB_URLS:
            jar_name = os.path.basename(url)
            jar_path = os.path.join(target_path, jar_name)
            if not os.path.exists(jar_path):
                print("Downloading java package:", jar_name)
                urlretrieve(url, jar_path)

    def run(self):
        self.download_jars(target_path=_PATH_TO_LIB)
        install.run(self)


setup(
    name="twkorean",
    license="Apache 2.0",
    version=__VERSION__,
    packages=["twkorean"],
    package_dir={"twkorean": "twkorean"},
    package_data={
        "twkorean": [
            "data/lib/*.jar",
        ],
    },
    install_requires=[
        "JPype1"
    ],
    author="Jaepil Jeong",
    author_email="jaepil@{nospam}appspand.com",
    url="https://github.com/jaepil/twkorean/",
    download_url="https://github.com/jaepil/twkorean/tree/master",
    classifiers=[
        "Programming Language :: Java",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
    ],
    platforms=[
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows :: Windows 7",
        "Operating System :: Microsoft :: Windows :: Windows Vista",
        "Operating System :: POSIX :: Linux",
    ],
    keywords=[
        "twitter-korean-text",
        "morphological analyzer",
        "morphology", "analyzer"
        "korean", "tokenizer"
    ],
    description="Python interface to twitter-korean-text, a Korean morphological analyzer.",
    cmdclass={
        'install': InstallCommand,
    }
)
